@@ -22,13 +22,13 @@
 
 #include "sourcesmodel.h"
 
-#include <QDebug>
 #include <KConfig>
-#include <KConfigGroup>
-#include <QTimer>
 #include <KDirWatch>
 #include <KSharedConfig>
 
+#include <QModelIndex>
+#include <QSet>
+
 using namespace Milou;
 
 SourcesModel::SourcesModel(QObject* parent)
@@ -221,11 +221,22 @@ void SourcesModel::slotMatchesChanged(const QList<Plasma::QueryMatch>& l)
     QSet<QString> higherTypes;
     for (QString type: m_types) {
         const TypeData td = m_matches.value(type);
-        if (!td.shown.isEmpty()) {
-            const auto& match = td.shown.first();
-            if (match.text().contains(m_queryString, Qt::CaseInsensitive)) {
-                QString matchType = match.matchCategory();
-                higherTypes << matchType;
+        for (const Plasma::QueryMatch& match : td.shown) {
+            const QString text = match.text().simplified();
+            const QStringList words = m_queryString.split(' ', QString::SkipEmptyParts);
+            bool containsAll = true;
+
+            for (const QString& word : words) {
+                if (!text.contains(word.simplified(), Qt::CaseInsensitive)) {
+                    containsAll = false;
+                    break;
+                }
+            }
+
+            // Maybe we should be giving it a higher type based on the number of matched
+            // words in the text?
+            if (containsAll) {
+                higherTypes << match.matchCategory();
             }
         }
     }
@@ -335,13 +346,39 @@ void SourcesModel::clear()
     m_duplicates.clear();
     m_queryString.clear();
     m_manager->reset();
+    m_manager->matchSessionComplete();
     endResetModel();
 }
 
-void SourcesModel::run(int index)
+bool SourcesModel::run(int index)
 {
     Plasma::QueryMatch match = fetchMatch(index);
+    Q_ASSERT(match.runner());
+
+    if (match.type() == Plasma::QueryMatch::InformationalMatch) {
+        QString info = match.data().toString();
+        int editPos = info.length();
+
+        if (!info.isEmpty()) {
+            // FIXME: pretty lame way to decide if this is a query prototype
+            // Copied from kde4 krunner interface.cpp
+            if (match.runner() == 0) {
+                // lame way of checking to see if this is a Help Button generated match!
+                int index = info.indexOf(QStringLiteral(":q:"));
+
+                if (index != -1) {
+                    editPos = index;
+                    info.replace(QStringLiteral(":q:"), QString());
+                }
+            }
+
+            emit updateSearchTerm(info, editPos);
+            return false;
+        }
+    }
+
     m_manager->run(match);
+    return true;
 }
 
 void SourcesModel::reloadConfiguration()
